#!/usr/bin/python
# Filename: dieroller.py
# Author: Tim Doty
# Version: v0.2.2 20080718

import wx
import math
import random

ID_ABOUT   = 101
ID_OPEN    = 102
ID_BUTTON1 = 110
ID_EXIT    = 110

class RowSet():
    """ Provides dice and row handling """
    def __init__(self, panel, numRows=1):
        self.maxRows = 10
        self.panel = panel
        self.window = panel.GetTopLevelParent()
        self.sizer = panel.GetSizer()
        #
        # possible operators. to use them would need to do a preprocessing to translate
        # into single char occurances
        #  log, ln
        #  sin, cos, tan, asin, acos, atan
        #  round, int, floor, ceil
        #  ?:
        #  adjSTR (results in die roll for indicated adjSTR)
        ##
        # a : adj (1->1, 2->1, 3->2, etc.)
        # p : points for adj (1->1, 2->3, 3->6, etc.)
        self.maxPrecedence = 99
        self.operatorPrecedence = '(aApds^*/+-)'
        self.operatorUnary = '(aAp'
        self.scaled = {}
        self.scaled[6] = [1, 1, 1, 2, 2, 3]
        self.scaled[8] = [1, 1, 1, 2, 2, 2, 3, 3]
        self.scaled[10] = [1, 1, 1, 1, 2, 2, 2, 3, 3, 4]
        self.scaled[12] = [1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 4, 4]
        self.scaled[20] = [1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 5, 5, 6]
        #
    def Tokenize(self, diceToRoll):
        diceToRoll = diceToRoll.replace('adjSTR', 'A')
        diceToRoll = diceToRoll.replace('adj', 'a')
        diceToRoll = diceToRoll.replace('points', 'p')
        diceToRoll = diceToRoll.replace('point', 'p')
        diceToRoll = diceToRoll.replace('pts', 'p')
        diceToRoll = diceToRoll.replace('pt', 'p')
        if diceToRoll == '':
            return [0]
        # tokenize the string
        parts = []
        appending = False
        for char in diceToRoll:
            if char in self.operatorPrecedence:
                parts += [char]
                appending = False
            else:
                if appending:
                   parts[-1] += char
                else:
                    parts += [char]
                    appending = True
        return parts
    def Evaluate(self, parts):
        self.stack = []
        self.lastOperator = self.maxPrecedence
        for token in parts:
            try:
                self.stack.append(int(token))
            except ValueError:
                currentPrecedence = self.operatorPrecedence.find(token)
                while self.lastOperator != 0 and self.lastOperator < currentPrecedence:
                    # lower precedence, let's do it!
                    self.__DoStackOperation()
                if token == ')':
                    if self.lastOperator == 0:
                        self.__DoStackOperation()
                    else:
                        print "ERROR: incomplete operations within parens"
                else:
                    self.stack.append(token)
                self.lastOperator = currentPrecedence
        # after going through all tokens make sure to collapse stack
        while len(self.stack) > 1:
            self.__DoStackOperation()
        return self.stack.pop()
    def DoRoll(self, event):
        # read the dice to roll, for each roll, and put result in the result
        # for now ignoring number in set
        # parse and validate dice
        # consider using MiniParser http://christophe.delord.free.fr/mp/index.html
        diceToRoll = self.inputDice.GetLineText(0) # always 0 (line number to return)
        parts = self.Tokenize(diceToRoll)
        # now interpret the results
        for result in self.results:
            dieRoll = self.Evaluate(parts)
            result.ChangeValue(str(dieRoll))
    def __DoStackOperation(self):
        # removes top three items on stack, performs operation, puts result back onto stack
        operandRight = self.stack.pop()
        operator = self.stack.pop()
        if self.operatorUnary.find(operator) != -1:
            # which unary operation?
            if operator == '(':
                self.stack.append(operandRight)
            elif operator == 'a':
                # adj (scaled number)
                self.stack.append( int((math.sqrt(8 * operandRight + 1) - 1) / 2) )
            elif operator == 'p':
                # inverse of scaled number
                self.stack.append( (operandRight * operandRight + operandRight) / 2 )
            elif operator == 'A':
                # adjSTR
                if operandRight < 0.10:
                    self.stack.append( 0 )
                elif operandRight < 0.50:
                    self.stack.append( random.randint(0, 1) )
                elif operandRight < 1.06:
                    self.stack.append( random.randint(0, 2) )
                elif operandRight < 1.75:
                    self.stack.append( random.randint(0, 3) )
                elif operandRight < 2.56:
                    self.stack.append( random.randint(1, 3) )
                elif operandRight < 3.93:
                    self.stack.append( random.randint(1, 4) )
                elif operandRight < 6.05:
                    self.stack.append( random.randint(1, 6) )
                elif operandRight < 8.45:
                    self.stack.append( random.randint(1, 8) )
                elif operandRight < 11.11:
                    self.stack.append( random.randint(1, 10) )
                elif operandRight < 13.26:
                    self.stack.append( random.randint(1, 12) )
                elif operandRight < 15.53:
                    roll = random.randint(1, 6)
                    self.stack.append( roll + random.randint(1, 6) )
                elif operandRight < 18.73:
                    roll = random.randint(1, 6)
                    self.stack.append( roll + random.randint(1, 8) )
                elif operandRight < 22.13:
                    roll = random.randint(1, 8)
                    self.stack.append( roll + random.randint(1, 8) )
                elif operandRight < 25.72:
                    roll = random.randint(1, 8)
                    self.stack.append( roll + random.randint(1, 10) )
                elif operandRight < 29.48:
                    roll = random.randint(1, 10)
                    self.stack.append( roll + random.randint(1, 10) )
                elif operandRight < 33.40:
                    roll = random.randint(1, 10)
                    self.stack.append( roll + random.randint(1, 12) )
                elif operandRight < 36.45:
                    roll = random.randint(1, 12)
                    self.stack.append( roll + random.randint(1, 12) )
                elif operandRight < 39.59:
                    roll = random.randint(1, 8)
                    roll += random.randint(1, 8)
                    self.stack.append( roll + random.randint(1, 8) )
                elif operandRight < 43.91:
                    roll = random.randint(1, 8)
                    roll += random.randint(1, 8)
                    self.stack.append( roll + random.randint(1, 10) )
                elif operandRight < 48.37:
                    roll = random.randint(1, 8)
                    roll += random.randint(1, 10)
                    self.stack.append( roll + random.randint(1, 10) )
                elif operandRight < 52.97:
                    roll = random.randint(1, 10)
                    roll += random.randint(1, 10)
                    self.stack.append( roll + random.randint(1, 10) )
                elif operandRight < 57.72:
                    roll = random.randint(1, 10)
                    roll += random.randint(1, 10)
                    self.stack.append( roll + random.randint(1, 12) )
                elif operandRight < 62.59:
                    roll = random.randint(1, 10)
                    roll += random.randint(1, 12)
                    self.stack.append( roll + random.randint(1, 12) )
                elif operandRight < 66.33:
                    roll = random.randint(1, 12)
                    roll += random.randint(1, 12)
                    self.stack.append( roll + random.randint(1, 12) )
                elif operandRight < 70.15:
                    roll = random.randint(1, 8)
                    roll += random.randint(1, 8)
                    roll += random.randint(1, 10)
                    self.stack.append( roll + random.randint(1, 10) )
                elif operandRight < 75.34:
                    roll = random.randint(1, 8)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 10)
                    self.stack.append( roll + random.randint(1, 10) )
                elif operandRight < 80.66:
                    roll = random.randint(1, 10)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 10)
                    self.stack.append( roll + random.randint(1, 10) )
                elif operandRight < 86.09:
                    roll = random.randint(1, 10)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 10)
                    self.stack.append( roll + random.randint(1, 12) )
                elif operandRight < 91.65:
                    roll = random.randint(1, 10)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 12)
                    self.stack.append( roll + random.randint(1, 12) )
                elif operandRight < 97.32:
                    roll = random.randint(1, 10)
                    roll += random.randint(1, 12)
                    roll += random.randint(1, 12)
                    self.stack.append( roll + random.randint(1, 12) )
                elif operandRight < 101.64:
                    roll = random.randint(1, 12)
                    roll += random.randint(1, 12)
                    roll += random.randint(1, 12)
                    self.stack.append( roll + random.randint(1, 12) )
                elif operandRight < 106.03:
                    roll = random.randint(1, 8)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 10)
                    self.stack.append( roll + random.randint(1, 10) )
                elif operandRight < 111.97:
                    roll = random.randint(1, 10)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 10)
                    self.stack.append( roll + random.randint(1, 10) )
                elif operandRight < 118.02:
                    roll = random.randint(1, 10)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 10)
                    self.stack.append( roll + random.randint(1, 12) )
                elif operandRight < 124.18:
                    roll = random.randint(1, 10)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 12)
                    self.stack.append( roll + random.randint(1, 12) )
                elif operandRight < 130.44:
                    roll = random.randint(1, 10)
                    roll += random.randint(1, 10)
                    roll += random.randint(1, 12)
                    roll += random.randint(1, 12)
                    self.stack.append( roll + random.randint(1, 12) )
                elif operandRight < 136.80:
                    roll = random.randint(1, 10)
                    roll += random.randint(1, 12)
                    roll += random.randint(1, 12)
                    roll += random.randint(1, 12)
                    self.stack.append( roll + random.randint(1, 12) )
                else:
                    # the really high adjSTR
                    avgDmg = math.pow(operandRight / .7557, 2/3)
                    numD12 = int(avgDmg / 6.5 + 0.5)
                    roll = 0
                    while numD12 > 0:
                        roll += random.randint(1, 12)
                        numD12 -= 1
                    self.stack.append(roll)
            else:
                print "ERROR: unknown unary operator '", operator, "'"
        else:
            # get first operand
            operandLeft = self.stack.pop()
            # which binary operation?
            if operator == 'd':
                roll = 0
                while operandLeft > 0:
                    roll += random.randint(1, operandRight)
                    operandLeft -= 1
                self.stack.append(roll)
            elif operator == 's':
                roll = 0
                while operandLeft > 0:
                    result = random.randint(1, operandRight)
                    try:
                        roll += self.scaled[operandRight][result-1]
                    except KeyError:
                        print "ERROR: not a valid scaled die size '", operandRight, "'"
                    operandLeft -= 1
                self.stack.append(roll)
            elif operator == '+':
                self.stack.append(operandLeft + operandRight)
            elif operator == '-':
                self.stack.append(operandLeft - operandRight)
            elif operator == '*':
                self.stack.append(operandLeft * operandRight)
            elif operator == '/':
                self.stack.append(operandLeft / operandRight)
            elif operator == '^':
                self.stack.append(int(math.pow(operandLeft, operandRight)))
            else:
                print "ERROR: unknown binary operator '", operator, "'"
                self.stack.append(operandLeft)
        # update "previous" operator precedence
        if len(self.stack) > 1:
            self.lastOperator = self.operatorPrecedence.find(self.stack[-2])
        else:
            self.lastOperator = self.maxPrecedence

class DiceSet(RowSet):
    """ Provides a set of arbitrary dice """
    def __init__(self, panel, indexBase=3, numRows=1):
        RowSet.__init__(self, panel=panel, numRows=numRows)
        self.results = [] # list of widgets for result output
        self.inputRows = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
        self.inputDice = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
        self.sizerResult = wx.BoxSizer(wx.VERTICAL)
        self.inputRows.ChangeValue(str(numRows))
        self.sizer.Insert(indexBase + 0, self.inputRows)
        self.sizer.Insert(indexBase + 1, self.inputDice)
        self.sizer.Insert(indexBase + 2, self.sizerResult)
        self.inputRows.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        self.inputDice.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        for row in range(0, numRows):
            result = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER | wx.TE_READONLY)
            self.results.append(result)
            self.sizerResult.Add(result)
    def Destroy(self):
        for row in range(0, len(self.results)):
            self.sizerResult.Remove(row)
            item = self.results.pop()
            item.Destroy()
        self.sizer.Remove(self.inputRows)
        self.sizer.Remove(self.inputDice)
        self.sizer.Remove(self.sizerResult)
        self.inputRows.Destroy()
        self.inputDice.Destroy()
        # need to trigger a re-layout
        self.window.Layout()
    def UpdateSet(self, event):
        try:
            numRows = int(self.inputRows.GetLineText(0)) # always 0 (line number to return)
            if numRows < 1:
                numRows = 1
            elif numRows > self.maxRows: # sanity check to avoid huge sizes
                numRows = self.maxRows
            self.inputRows.ChangeValue(str(numRows))
        except ValueError:
            numRows = len(self.results)
            self.inputRows.ChangeValue(numRows)
        if numRows < len(self.results):
            # need to remove some result rows
            # removes from sizer and result stack in opposite orders, but since it
            # is the same number of items and the removals are independent this is
            # not an issue
            for row in range( numRows, len(self.results) ):
                self.sizerResult.Remove(row)
                item = self.results.pop()
                item.Destroy()
            # need to trigger a re-layout
            self.window.Layout()
        elif numRows > len(self.results):
            # need to add some result rows
            for row in range( len(self.results), numRows ):
                result = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER | wx.TE_READONLY)
                self.results.append(result)
                self.sizerResult.Add(result)
            # probably need to trigger a re-layout?
            self.window.Layout()
        self.DoRoll(event)

class SkillSet(RowSet):
    """ Provides handling for skill evaluation """
    def __init__(self, panel, indexBase=7, numRows=1):
        RowSet.__init__(self, panel=panel, numRows=numRows)
        self.rolls = [] # list of widgets for roll output
        self.results = [] # list of widgets for result output
        self.inputRows = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
        self.inputScore = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
        self.inputDifficulty = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
        self.inputAdj = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
        self.inputChance = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER | wx.TE_READONLY)
        self.sizerRoll = wx.BoxSizer(wx.VERTICAL)
        self.sizerResult = wx.BoxSizer(wx.VERTICAL)
        self.inputRows.ChangeValue(str(numRows))
        self.sizer.Insert(indexBase + 0, self.inputRows)
        self.sizer.Insert(indexBase + 1, self.inputScore)
        self.sizer.Insert(indexBase + 2, self.inputDifficulty)
        self.sizer.Insert(indexBase + 3, self.inputAdj)
        self.sizer.Insert(indexBase + 4, self.inputChance)
        self.sizer.Insert(indexBase + 5, self.sizerRoll)
        self.sizer.Insert(indexBase + 6, self.sizerResult)
        self.inputRows.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        self.inputScore.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        self.inputDifficulty.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        self.inputAdj.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        self.inputChance.Bind(wx.EVT_TEXT_ENTER, self.UpdateSet)
        for row in range(0, numRows):
            roll = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
            roll.Bind(wx.EVT_TEXT_ENTER, self.UpdateResult)
            self.rolls.append(roll)
            self.sizerRoll.Add(roll)
            result = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER | wx.TE_READONLY)
            self.results.append(result)
            self.sizerResult.Add(result)
        # Level of Success Table
        self.Critical3 = {}
        self.Critical2 = {}
        self.Critical = {}
        self.Special = {}
        self.Miss = {}
        self.Failure = {}
        self.Fumble = {}
        self.Fumble2 = {}
        self.Critical3[0] = 0
        self.Critical3[1] = 0
        self.Critical3[2] = 0
        self.Critical3[3] = 0
        self.Critical3[4] = 0
        self.Critical3[5] = 0
        self.Critical3[6] = 0
        self.Critical3[7] = 0
        self.Critical3[8] = 0
        self.Critical3[9] = 0
        self.Critical3[10] = 0
        self.Critical3[11] = 0
        self.Critical3[12] = 0
        self.Critical3[13] = 0
        self.Critical3[14] = 0
        self.Critical3[15] = 0
        self.Critical3[16] = 0
        self.Critical3[17] = 0
        self.Critical3[18] = 0
        self.Critical3[19] = 0
        self.Critical3[20] = 0
        self.Critical3[21] = 0
        self.Critical3[22] = 0
        self.Critical3[23] = 0
        self.Critical3[24] = 0
        self.Critical3[25] = 0
        self.Critical3[26] = 0
        self.Critical3[27] = 0
        self.Critical3[28] = 0
        self.Critical3[29] = 0
        self.Critical3[30] = 0
        self.Critical3[31] = 0
        self.Critical3[32] = 0
        self.Critical3[33] = 0
        self.Critical3[34] = 0
        self.Critical3[35] = 0
        self.Critical3[36] = 0
        self.Critical3[37] = 0
        self.Critical3[38] = 0
        self.Critical3[39] = 0
        self.Critical3[40] = 0
        self.Critical3[41] = 0
        self.Critical3[42] = 0
        self.Critical3[43] = 0
        self.Critical3[44] = 0
        self.Critical3[45] = 0
        self.Critical3[46] = 0
        self.Critical3[47] = 0
        self.Critical3[48] = 0
        self.Critical3[49] = 0
        self.Critical3[50] = 0
        self.Critical3[51] = 0
        self.Critical3[52] = 0
        self.Critical3[53] = 0
        self.Critical3[54] = 0
        self.Critical3[55] = 0
        self.Critical3[56] = 0
        self.Critical3[57] = 0
        self.Critical3[58] = 0
        self.Critical3[59] = 0
        self.Critical3[60] = 0
        self.Critical3[61] = 0
        self.Critical3[62] = 0
        self.Critical3[63] = 0
        self.Critical3[64] = 0
        self.Critical3[65] = 0
        self.Critical3[66] = 0
        self.Critical3[67] = 0
        self.Critical3[68] = 0
        self.Critical3[69] = 0
        self.Critical3[70] = 0
        self.Critical3[71] = 0
        self.Critical3[72] = 0
        self.Critical3[73] = 0
        self.Critical3[74] = 0
        self.Critical3[75] = 0
        self.Critical3[76] = 0
        self.Critical3[77] = 0
        self.Critical3[78] = 0
        self.Critical3[79] = 0
        self.Critical3[80] = 0
        self.Critical3[81] = 0
        self.Critical3[82] = 0
        self.Critical3[83] = 0
        self.Critical3[84] = 0
        self.Critical3[85] = 0
        self.Critical3[86] = 0
        self.Critical3[87] = 0
        self.Critical3[88] = 0
        self.Critical3[89] = 0
        self.Critical3[90] = 0
        self.Critical3[91] = 0
        self.Critical3[92] = 0
        self.Critical3[93] = 1
        self.Critical3[94] = 1
        self.Critical3[95] = 1
        self.Critical3[96] = 1
        self.Critical3[97] = 2
        self.Critical3[98] = 3
        self.Critical3[99] = 5
        self.Critical3[100] = 8
        self.Critical2[0] = 0
        self.Critical2[1] = 0
        self.Critical2[2] = 0
        self.Critical2[3] = 0
        self.Critical2[4] = 0
        self.Critical2[5] = 0
        self.Critical2[6] = 0
        self.Critical2[7] = 0
        self.Critical2[8] = 0
        self.Critical2[9] = 0
        self.Critical2[10] = 0
        self.Critical2[11] = 0
        self.Critical2[12] = 0
        self.Critical2[13] = 0
        self.Critical2[14] = 0
        self.Critical2[15] = 0
        self.Critical2[16] = 0
        self.Critical2[17] = 0
        self.Critical2[18] = 0
        self.Critical2[19] = 0
        self.Critical2[20] = 0
        self.Critical2[21] = 0
        self.Critical2[22] = 0
        self.Critical2[23] = 0
        self.Critical2[24] = 0
        self.Critical2[25] = 0
        self.Critical2[26] = 0
        self.Critical2[27] = 0
        self.Critical2[28] = 0
        self.Critical2[29] = 0
        self.Critical2[30] = 0
        self.Critical2[31] = 0
        self.Critical2[32] = 0
        self.Critical2[33] = 0
        self.Critical2[34] = 0
        self.Critical2[35] = 0
        self.Critical2[36] = 0
        self.Critical2[37] = 0
        self.Critical2[38] = 0
        self.Critical2[39] = 0
        self.Critical2[40] = 0
        self.Critical2[41] = 0
        self.Critical2[42] = 0
        self.Critical2[43] = 0
        self.Critical2[44] = 0
        self.Critical2[45] = 0
        self.Critical2[46] = 0
        self.Critical2[47] = 0
        self.Critical2[48] = 0
        self.Critical2[49] = 0
        self.Critical2[50] = 0
        self.Critical2[51] = 0
        self.Critical2[52] = 0
        self.Critical2[53] = 0
        self.Critical2[54] = 0
        self.Critical2[55] = 0
        self.Critical2[56] = 0
        self.Critical2[57] = 0
        self.Critical2[58] = 0
        self.Critical2[59] = 0
        self.Critical2[60] = 0
        self.Critical2[61] = 0
        self.Critical2[62] = 0
        self.Critical2[63] = 0
        self.Critical2[64] = 0
        self.Critical2[65] = 0
        self.Critical2[66] = 0
        self.Critical2[67] = 1
        self.Critical2[68] = 1
        self.Critical2[69] = 1
        self.Critical2[70] = 1
        self.Critical2[71] = 1
        self.Critical2[72] = 1
        self.Critical2[73] = 1
        self.Critical2[74] = 1
        self.Critical2[75] = 1
        self.Critical2[76] = 1
        self.Critical2[77] = 1
        self.Critical2[78] = 1
        self.Critical2[79] = 1
        self.Critical2[80] = 2
        self.Critical2[81] = 2
        self.Critical2[82] = 2
        self.Critical2[83] = 2
        self.Critical2[84] = 2
        self.Critical2[85] = 2
        self.Critical2[86] = 3
        self.Critical2[87] = 3
        self.Critical2[88] = 3
        self.Critical2[89] = 4
        self.Critical2[90] = 4
        self.Critical2[91] = 5
        self.Critical2[92] = 6
        self.Critical2[93] = 6
        self.Critical2[94] = 7
        self.Critical2[95] = 9
        self.Critical2[96] = 11
        self.Critical2[97] = 13
        self.Critical2[98] = 17
        self.Critical2[99] = 25
        self.Critical2[100] = 34
        self.Critical[0] = 0
        self.Critical[1] = 0
        self.Critical[2] = 0
        self.Critical[3] = 0
        self.Critical[4] = 0
        self.Critical[5] = 0
        self.Critical[6] = 0
        self.Critical[7] = 0
        self.Critical[8] = 0
        self.Critical[9] = 0
        self.Critical[10] = 0
        self.Critical[11] = 0
        self.Critical[12] = 0
        self.Critical[13] = 0
        self.Critical[14] = 0
        self.Critical[15] = 0
        self.Critical[16] = 0
        self.Critical[17] = 0
        self.Critical[18] = 0
        self.Critical[19] = 0
        self.Critical[20] = 0
        self.Critical[21] = 0
        self.Critical[22] = 0
        self.Critical[23] = 0
        self.Critical[24] = 0
        self.Critical[25] = 0
        self.Critical[26] = 0
        self.Critical[27] = 0
        self.Critical[28] = 0
        self.Critical[29] = 1
        self.Critical[30] = 1
        self.Critical[31] = 1
        self.Critical[32] = 1
        self.Critical[33] = 1
        self.Critical[34] = 1
        self.Critical[35] = 1
        self.Critical[36] = 1
        self.Critical[37] = 1
        self.Critical[38] = 1
        self.Critical[39] = 1
        self.Critical[40] = 1
        self.Critical[41] = 1
        self.Critical[42] = 1
        self.Critical[43] = 1
        self.Critical[44] = 2
        self.Critical[45] = 2
        self.Critical[46] = 2
        self.Critical[47] = 2
        self.Critical[48] = 2
        self.Critical[49] = 2
        self.Critical[50] = 2
        self.Critical[51] = 2
        self.Critical[52] = 3
        self.Critical[53] = 3
        self.Critical[54] = 3
        self.Critical[55] = 3
        self.Critical[56] = 3
        self.Critical[57] = 3
        self.Critical[58] = 4
        self.Critical[59] = 4
        self.Critical[60] = 4
        self.Critical[61] = 4
        self.Critical[62] = 5
        self.Critical[63] = 5
        self.Critical[64] = 5
        self.Critical[65] = 5
        self.Critical[66] = 6
        self.Critical[67] = 6
        self.Critical[68] = 6
        self.Critical[69] = 7
        self.Critical[70] = 7
        self.Critical[71] = 7
        self.Critical[72] = 8
        self.Critical[73] = 8
        self.Critical[74] = 9
        self.Critical[75] = 9
        self.Critical[76] = 10
        self.Critical[77] = 10
        self.Critical[78] = 11
        self.Critical[79] = 12
        self.Critical[80] = 12
        self.Critical[81] = 13
        self.Critical[82] = 14
        self.Critical[83] = 15
        self.Critical[84] = 16
        self.Critical[85] = 17
        self.Critical[86] = 18
        self.Critical[87] = 19
        self.Critical[88] = 20
        self.Critical[89] = 22
        self.Critical[90] = 24
        self.Critical[91] = 25
        self.Critical[92] = 28
        self.Critical[93] = 30
        self.Critical[94] = 33
        self.Critical[95] = 36
        self.Critical[96] = 40
        self.Critical[97] = 45
        self.Critical[98] = 52
        self.Critical[99] = 63
        self.Critical[100] = 72
        self.Special[0] = 0
        self.Special[1] = 0
        self.Special[2] = 0
        self.Special[3] = 0
        self.Special[4] = 0
        self.Special[5] = 0
        self.Special[6] = 1
        self.Special[7] = 1
        self.Special[8] = 1
        self.Special[9] = 1
        self.Special[10] = 1
        self.Special[11] = 1
        self.Special[12] = 1
        self.Special[13] = 2
        self.Special[14] = 2
        self.Special[15] = 2
        self.Special[16] = 2
        self.Special[17] = 3
        self.Special[18] = 3
        self.Special[19] = 3
        self.Special[20] = 3
        self.Special[21] = 4
        self.Special[22] = 4
        self.Special[23] = 4
        self.Special[24] = 4
        self.Special[25] = 5
        self.Special[26] = 5
        self.Special[27] = 5
        self.Special[28] = 6
        self.Special[29] = 6
        self.Special[30] = 6
        self.Special[31] = 7
        self.Special[32] = 7
        self.Special[33] = 8
        self.Special[34] = 8
        self.Special[35] = 8
        self.Special[36] = 9
        self.Special[37] = 9
        self.Special[38] = 10
        self.Special[39] = 10
        self.Special[40] = 11
        self.Special[41] = 11
        self.Special[42] = 11
        self.Special[43] = 12
        self.Special[44] = 12
        self.Special[45] = 13
        self.Special[46] = 14
        self.Special[47] = 14
        self.Special[48] = 15
        self.Special[49] = 15
        self.Special[50] = 16
        self.Special[51] = 16
        self.Special[52] = 17
        self.Special[53] = 18
        self.Special[54] = 18
        self.Special[55] = 19
        self.Special[56] = 20
        self.Special[57] = 20
        self.Special[58] = 21
        self.Special[59] = 22
        self.Special[60] = 23
        self.Special[61] = 24
        self.Special[62] = 24
        self.Special[63] = 25
        self.Special[64] = 26
        self.Special[65] = 27
        self.Special[66] = 28
        self.Special[67] = 29
        self.Special[68] = 30
        self.Special[69] = 31
        self.Special[70] = 32
        self.Special[71] = 33
        self.Special[72] = 34
        self.Special[73] = 35
        self.Special[74] = 36
        self.Special[75] = 37
        self.Special[76] = 38
        self.Special[77] = 40
        self.Special[78] = 41
        self.Special[79] = 42
        self.Special[80] = 44
        self.Special[81] = 45
        self.Special[82] = 47
        self.Special[83] = 48
        self.Special[84] = 50
        self.Special[85] = 51
        self.Special[86] = 53
        self.Special[87] = 55
        self.Special[88] = 57
        self.Special[89] = 59
        self.Special[90] = 61
        self.Special[91] = 63
        self.Special[92] = 66
        self.Special[93] = 68
        self.Special[94] = 71
        self.Special[95] = 74
        self.Special[96] = 77
        self.Special[97] = 81
        self.Special[98] = 85
        self.Special[99] = 91
        self.Special[100] = 94
        self.Miss[0] = 6
        self.Miss[1] = 9
        self.Miss[2] = 15
        self.Miss[3] = 19
        self.Miss[4] = 23
        self.Miss[5] = 26
        self.Miss[6] = 29
        self.Miss[7] = 32
        self.Miss[8] = 34
        self.Miss[9] = 37
        self.Miss[10] = 39
        self.Miss[11] = 41
        self.Miss[12] = 43
        self.Miss[13] = 45
        self.Miss[14] = 47
        self.Miss[15] = 49
        self.Miss[16] = 50
        self.Miss[17] = 52
        self.Miss[18] = 53
        self.Miss[19] = 55
        self.Miss[20] = 56
        self.Miss[21] = 58
        self.Miss[22] = 59
        self.Miss[23] = 60
        self.Miss[24] = 62
        self.Miss[25] = 63
        self.Miss[26] = 64
        self.Miss[27] = 65
        self.Miss[28] = 66
        self.Miss[29] = 67
        self.Miss[30] = 68
        self.Miss[31] = 69
        self.Miss[32] = 70
        self.Miss[33] = 71
        self.Miss[34] = 72
        self.Miss[35] = 73
        self.Miss[36] = 74
        self.Miss[37] = 75
        self.Miss[38] = 76
        self.Miss[39] = 76
        self.Miss[40] = 77
        self.Miss[41] = 78
        self.Miss[42] = 79
        self.Miss[43] = 80
        self.Miss[44] = 80
        self.Miss[45] = 81
        self.Miss[46] = 82
        self.Miss[47] = 82
        self.Miss[48] = 83
        self.Miss[49] = 84
        self.Miss[50] = 84
        self.Miss[51] = 85
        self.Miss[52] = 85
        self.Miss[53] = 86
        self.Miss[54] = 86
        self.Miss[55] = 87
        self.Miss[56] = 88
        self.Miss[57] = 88
        self.Miss[58] = 89
        self.Miss[59] = 89
        self.Miss[60] = 89
        self.Miss[61] = 90
        self.Miss[62] = 90
        self.Miss[63] = 91
        self.Miss[64] = 91
        self.Miss[65] = 92
        self.Miss[66] = 92
        self.Miss[67] = 92
        self.Miss[68] = 93
        self.Miss[69] = 93
        self.Miss[70] = 94
        self.Miss[71] = 94
        self.Miss[72] = 94
        self.Miss[73] = 95
        self.Miss[74] = 95
        self.Miss[75] = 95
        self.Miss[76] = 96
        self.Miss[77] = 96
        self.Miss[78] = 96
        self.Miss[79] = 96
        self.Miss[80] = 97
        self.Miss[81] = 97
        self.Miss[82] = 97
        self.Miss[83] = 97
        self.Miss[84] = 98
        self.Miss[85] = 98
        self.Miss[86] = 98
        self.Miss[87] = 98
        self.Miss[88] = 99
        self.Miss[89] = 99
        self.Miss[90] = 99
        self.Miss[91] = 99
        self.Miss[92] = 99
        self.Miss[93] = 99
        self.Miss[94] = 99
        self.Miss[95] = 100
        self.Miss[96] = 100
        self.Miss[97] = 100
        self.Miss[98] = 100
        self.Miss[99] = 100
        self.Miss[100] = 101
        self.Failure[0] = 28
        self.Failure[1] = 37
        self.Failure[2] = 48
        self.Failure[3] = 55
        self.Failure[4] = 60
        self.Failure[5] = 64
        self.Failure[6] = 67
        self.Failure[7] = 70
        self.Failure[8] = 72
        self.Failure[9] = 75
        self.Failure[10] = 76
        self.Failure[11] = 78
        self.Failure[12] = 70
        self.Failure[13] = 81
        self.Failure[14] = 82
        self.Failure[15] = 83
        self.Failure[16] = 84
        self.Failure[17] = 85
        self.Failure[18] = 86
        self.Failure[19] = 87
        self.Failure[20] = 88
        self.Failure[21] = 88
        self.Failure[22] = 89
        self.Failure[23] = 90
        self.Failure[24] = 90
        self.Failure[25] = 91
        self.Failure[26] = 91
        self.Failure[27] = 92
        self.Failure[28] = 92
        self.Failure[29] = 93
        self.Failure[30] = 93
        self.Failure[31] = 93
        self.Failure[32] = 94
        self.Failure[33] = 94
        self.Failure[34] = 94
        self.Failure[35] = 95
        self.Failure[36] = 95
        self.Failure[37] = 95
        self.Failure[38] = 95
        self.Failure[39] = 96
        self.Failure[40] = 96
        self.Failure[41] = 96
        self.Failure[42] = 96
        self.Failure[43] = 97
        self.Failure[44] = 97
        self.Failure[45] = 97
        self.Failure[46] = 97
        self.Failure[47] = 97
        self.Failure[48] = 97
        self.Failure[49] = 98
        self.Failure[50] = 98
        self.Failure[51] = 98
        self.Failure[52] = 98
        self.Failure[53] = 98
        self.Failure[54] = 98
        self.Failure[55] = 98
        self.Failure[56] = 98
        self.Failure[57] = 99
        self.Failure[58] = 99
        self.Failure[59] = 99
        self.Failure[60] = 99
        self.Failure[61] = 99
        self.Failure[62] = 99
        self.Failure[63] = 99
        self.Failure[64] = 99
        self.Failure[65] = 99
        self.Failure[66] = 99
        self.Failure[67] = 99
        self.Failure[68] = 99
        self.Failure[69] = 99
        self.Failure[70] = 99
        self.Failure[71] = 99
        self.Failure[72] = 100
        self.Failure[73] = 100
        self.Failure[74] = 100
        self.Failure[75] = 100
        self.Failure[76] = 100
        self.Failure[77] = 100
        self.Failure[78] = 100
        self.Failure[79] = 100
        self.Failure[80] = 100
        self.Failure[81] = 100
        self.Failure[82] = 100
        self.Failure[83] = 100
        self.Failure[84] = 100
        self.Failure[85] = 100
        self.Failure[86] = 100
        self.Failure[87] = 100
        self.Failure[88] = 100
        self.Failure[89] = 100
        self.Failure[90] = 100
        self.Failure[91] = 100
        self.Failure[92] = 100
        self.Failure[93] = 100
        self.Failure[94] = 100
        self.Failure[95] = 101
        self.Failure[96] = 101
        self.Failure[97] = 101
        self.Failure[98] = 101
        self.Failure[99] = 101
        self.Failure[100] = 101
        self.Fumble[0] = 66
        self.Fumble[1] = 75
        self.Fumble[2] = 83
        self.Fumble[3] = 87
        self.Fumble[4] = 89
        self.Fumble[5] = 91
        self.Fumble[6] = 93
        self.Fumble[7] = 94
        self.Fumble[8] = 94
        self.Fumble[9] = 95
        self.Fumble[10] = 96
        self.Fumble[11] = 96
        self.Fumble[12] = 97
        self.Fumble[13] = 97
        self.Fumble[14] = 97
        self.Fumble[15] = 98
        self.Fumble[16] = 98
        self.Fumble[17] = 98
        self.Fumble[18] = 98
        self.Fumble[19] = 98
        self.Fumble[20] = 98
        self.Fumble[21] = 99
        self.Fumble[22] = 99
        self.Fumble[23] = 99
        self.Fumble[24] = 99
        self.Fumble[25] = 99
        self.Fumble[26] = 99
        self.Fumble[27] = 99
        self.Fumble[28] = 99
        self.Fumble[29] = 99
        self.Fumble[30] = 99
        self.Fumble[31] = 99
        self.Fumble[32] = 99
        self.Fumble[33] = 99
        self.Fumble[34] = 100
        self.Fumble[35] = 100
        self.Fumble[36] = 100
        self.Fumble[37] = 100
        self.Fumble[38] = 100
        self.Fumble[39] = 100
        self.Fumble[40] = 100
        self.Fumble[41] = 100
        self.Fumble[42] = 100
        self.Fumble[43] = 100
        self.Fumble[44] = 100
        self.Fumble[45] = 100
        self.Fumble[46] = 100
        self.Fumble[47] = 100
        self.Fumble[48] = 100
        self.Fumble[49] = 100
        self.Fumble[50] = 100
        self.Fumble[51] = 100
        self.Fumble[52] = 100
        self.Fumble[53] = 100
        self.Fumble[54] = 100
        self.Fumble[55] = 100
        self.Fumble[56] = 100
        self.Fumble[57] = 100
        self.Fumble[58] = 100
        self.Fumble[59] = 100
        self.Fumble[60] = 100
        self.Fumble[61] = 100
        self.Fumble[62] = 100
        self.Fumble[63] = 100
        self.Fumble[64] = 100
        self.Fumble[65] = 100
        self.Fumble[66] = 100
        self.Fumble[67] = 100
        self.Fumble[68] = 100
        self.Fumble[69] = 100
        self.Fumble[70] = 100
        self.Fumble[71] = 100
        self.Fumble[72] = 101
        self.Fumble[73] = 101
        self.Fumble[74] = 101
        self.Fumble[75] = 101
        self.Fumble[76] = 101
        self.Fumble[77] = 101
        self.Fumble[78] = 101
        self.Fumble[79] = 101
        self.Fumble[80] = 101
        self.Fumble[81] = 101
        self.Fumble[82] = 101
        self.Fumble[83] = 101
        self.Fumble[84] = 101
        self.Fumble[85] = 101
        self.Fumble[86] = 101
        self.Fumble[87] = 101
        self.Fumble[88] = 101
        self.Fumble[89] = 101
        self.Fumble[90] = 101
        self.Fumble[91] = 101
        self.Fumble[92] = 101
        self.Fumble[93] = 101
        self.Fumble[94] = 101
        self.Fumble[95] = 101
        self.Fumble[96] = 101
        self.Fumble[97] = 101
        self.Fumble[98] = 101
        self.Fumble[99] = 101
        self.Fumble[100] = 101
        self.Fumble2[0] = 92
        self.Fumble2[1] = 95
        self.Fumble2[2] = 97
        self.Fumble2[3] = 98
        self.Fumble2[4] = 99
        self.Fumble2[5] = 99
        self.Fumble2[6] = 99
        self.Fumble2[7] = 99
        self.Fumble2[8] = 100
        self.Fumble2[9] = 100
        self.Fumble2[10] = 100
        self.Fumble2[11] = 100
        self.Fumble2[12] = 100
        self.Fumble2[13] = 100
        self.Fumble2[14] = 100
        self.Fumble2[15] = 100
        self.Fumble2[16] = 100
        self.Fumble2[17] = 100
        self.Fumble2[18] = 100
        self.Fumble2[19] = 100
        self.Fumble2[20] = 100
        self.Fumble2[21] = 100
        self.Fumble2[22] = 100
        self.Fumble2[23] = 100
        self.Fumble2[24] = 100
        self.Fumble2[25] = 100
        self.Fumble2[26] = 100
        self.Fumble2[27] = 100
        self.Fumble2[28] = 100
        self.Fumble2[29] = 100
        self.Fumble2[30] = 100
        self.Fumble2[31] = 100
        self.Fumble2[32] = 100
        self.Fumble2[33] = 100
        self.Fumble2[34] = 101
        self.Fumble2[35] = 101
        self.Fumble2[36] = 101
        self.Fumble2[37] = 101
        self.Fumble2[38] = 101
        self.Fumble2[39] = 101
        self.Fumble2[40] = 101
        self.Fumble2[41] = 101
        self.Fumble2[42] = 101
        self.Fumble2[43] = 101
        self.Fumble2[44] = 101
        self.Fumble2[45] = 101
        self.Fumble2[46] = 101
        self.Fumble2[47] = 101
        self.Fumble2[48] = 101
        self.Fumble2[49] = 101
        self.Fumble2[50] = 101
        self.Fumble2[51] = 101
        self.Fumble2[52] = 101
        self.Fumble2[53] = 101
        self.Fumble2[54] = 101
        self.Fumble2[55] = 101
        self.Fumble2[56] = 101
        self.Fumble2[57] = 101
        self.Fumble2[58] = 101
        self.Fumble2[59] = 101
        self.Fumble2[60] = 101
        self.Fumble2[61] = 101
        self.Fumble2[62] = 101
        self.Fumble2[63] = 101
        self.Fumble2[64] = 101
        self.Fumble2[65] = 101
        self.Fumble2[66] = 101
        self.Fumble2[67] = 101
        self.Fumble2[68] = 101
        self.Fumble2[69] = 101
        self.Fumble2[70] = 101
        self.Fumble2[71] = 101
        self.Fumble2[72] = 101
        self.Fumble2[73] = 101
        self.Fumble2[74] = 101
        self.Fumble2[75] = 101
        self.Fumble2[76] = 101
        self.Fumble2[77] = 101
        self.Fumble2[78] = 101
        self.Fumble2[79] = 101
        self.Fumble2[80] = 101
        self.Fumble2[81] = 101
        self.Fumble2[82] = 101
        self.Fumble2[83] = 101
        self.Fumble2[84] = 101
        self.Fumble2[85] = 101
        self.Fumble2[86] = 101
        self.Fumble2[87] = 101
        self.Fumble2[88] = 101
        self.Fumble2[89] = 101
        self.Fumble2[90] = 101
        self.Fumble2[91] = 101
        self.Fumble2[92] = 101
        self.Fumble2[93] = 101
        self.Fumble2[94] = 101
        self.Fumble2[95] = 101
        self.Fumble2[96] = 101
        self.Fumble2[97] = 101
        self.Fumble2[98] = 101
        self.Fumble2[99] = 101
        self.Fumble2[100] = 101
    def Destroy(self):
        for row in range(0, len(self.results)):
            self.sizerRoll.Remove(row)
            item = self.rolls.pop()
            item.Destroy()
            self.sizerResult.Remove(row)
            item = self.results.pop()
            item.Destroy()
        self.sizer.Remove(self.inputRows)
        self.sizer.Remove(self.inputScore)
        self.sizer.Remove(self.inputDifficulty)
        self.sizer.Remove(self.inputAdj)
        self.sizer.Remove(self.inputChance)
        self.sizer.Remove(self.sizerRoll)
        self.sizer.Remove(self.sizerResult)
        self.inputRows.Destroy()
        self.inputScore.Destroy()
        self.inputDifficulty.Destroy()
        self.inputAdj.Destroy()
        self.inputChance.Destroy()
        # need to trigger a re-layout
        self.window.Layout()
    def UpdateSet(self, event):
        try:
            numRows = int(self.inputRows.GetLineText(0)) # always 0 (line number to return)
            if numRows < 1:
                numRows = 1
            elif numRows > self.maxRows: # sanity check to avoid huge sizes
                numRows = self.maxRows
            self.inputRows.ChangeValue(str(numRows))
        except ValueError:
            numRows = len(self.results)
            self.inputRows.ChangeValue(numRows)
        if numRows < len(self.results):
            # need to remove some result rows
            # removes from sizer and result stack in opposite orders, but since it
            # is the same number of items and the removals are independent this is
            # not an issue
            for row in range( numRows, len(self.results) ):
                self.sizerRoll.Remove(row)
                self.sizerResult.Remove(row)
                item = self.rolls.pop()
                item.Destroy()
                item = self.results.pop()
                item.Destroy()
            # need to trigger a re-layout
            self.window.Layout()
        elif numRows > len(self.results):
            # need to add some result rows
            for row in range( len(self.results), numRows ):
                roll = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER)
                self.rolls.append(roll)
                self.sizerRoll.Add(roll)
                result = wx.TextCtrl(self.panel, style=wx.TE_PROCESS_ENTER | wx.TE_READONLY)
                self.results.append(result)
                self.sizerResult.Add(result)
            # probably need to trigger a re-layout?
            self.window.Layout()
        self.DoSkillRoll(event)
    def UpdateResult(self, event):
        chance = int(self.inputChance.GetLineText(0))
        roll = int(self.Evaluate( self.Tokenize( self.rolls[0].GetLineText(0) ) ))
        self.rolls[0].ChangeValue(str(roll))
        result = self.GetResult(chance, roll)
        self.results[0].ChangeValue(result)
    def DoSkillRoll(self, event):
        score = float(self.Evaluate( self.Tokenize( self.inputScore.GetLineText(0) ) ))
        self.inputScore.ChangeValue(str(int(score)))
        difficulty = float(self.Evaluate( self.Tokenize( self.inputDifficulty.GetLineText(0) ) ))
        self.inputDifficulty.ChangeValue(str(int(difficulty)))
        adj = float(self.Evaluate( self.Tokenize( self.inputAdj.GetLineText(0) ) ))
        self.inputAdj.ChangeValue(str(int(adj)))
        if score == 0:
            chance = 0
        elif difficulty == 0:
            chance = 100
        elif score > difficulty:
            chance = 100 - int((difficulty * difficulty) / (score * score) * 50 + .5) + int(adj)
        elif score < difficulty:
            chance = int((score * score) / (difficulty * difficulty) * 50 + .5) + int(adj)
        else:
            chance = 50
        self.inputChance.ChangeValue(str(chance))
        for row in range(0, len(self.results)):
            roll = random.randint(1, 100)
            result = self.GetResult(chance, roll)
            self.rolls[row].ChangeValue(str(roll))
            self.results[row].ChangeValue(result)
    def GetResult(self, chance, roll):
        if chance < 0:
            roll -= chance
            chance = 0
        elif chance > 100:
            roll = roll - (chance - 100)
            chance = 100
        if chance == 100 and roll < 2:
            result = 'Critical4'
        elif self.Critical3[chance] >= roll:
            result = 'Critical3'
        elif self.Critical2[chance] >= roll:
            result = 'Critical2'
        elif self.Critical[chance] >= roll:
            result = 'Critical'
        elif self.Special[chance] >= roll:
            result = 'Special'
        elif chance >= roll:
            result = 'Normal'
        elif self.Miss[chance] >= roll:
            result = 'Miss'
        elif self.Failure[chance] >= roll:
            result = 'Failure'
        elif self.Fumble[chance] >= roll:
            result = 'Fumble'
        elif self.Fumble2[chance] >= roll:
            result = 'Fumble2'
        elif chance == 0 and roll >= 100:
            result = 'Fumble4'
        else:
            result = 'Fumble3'
        return result

class MainWindow(wx.Frame):
    """ Dice Roller's main window is simply a subclass of Frame """
    def __init__(self, parent, id, title):
        wx.Frame.__init__(self, parent, id, title, size=(-1,-1), style=wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL)
        self.DiceSets = []
        self.SkillSets = []
        ## Layout
        #   Number     Dice     Result
        # [        ] [       ] [       ]
        # [Roll All] [Add Set] [Del Set]
        self.sizerMain = wx.FlexGridSizer()
        self.sizerMain.SetFlexibleDirection(wx.BOTH)
        self.panelDice = wx.Panel(self)
        self.sizerMain.Add(self.panelDice, flag=wx.EXPAND)
        self.sizerDice = wx.FlexGridSizer(cols=3)
        self.sizerDice.SetFlexibleDirection(wx.BOTH)
        headNumber = wx.StaticText(self.panelDice, label='Number', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headDice = wx.StaticText(self.panelDice, label='Dice', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headResult = wx.StaticText(self.panelDice, label='Result', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        self.buttonRoll = wx.Button(self.panelDice, label='Roll All')
        self.buttonAddSet = wx.Button(self.panelDice, label='Add Set')
        self.buttonDelSet = wx.Button(self.panelDice, label='Del Set')
        self.sizerDice.Add(headNumber)
        self.sizerDice.Add(headDice)
        self.sizerDice.Add(headResult)
        self.sizerDice.Add(self.buttonRoll)
        self.sizerDice.Add(self.buttonAddSet)
        self.sizerDice.Add(self.buttonDelSet)
        self.panelDice.SetSizer(self.sizerDice)
        self.panelDice.SetAutoLayout(True)
        self.AddDiceSet(False)
        #
        self.panelSkills = wx.Panel(self)
        self.sizerMain.Add(self.panelSkills)
        self.sizerSkills = wx.FlexGridSizer(cols=7)
        self.sizerDice.SetFlexibleDirection(wx.BOTH)
        headNumSkills = wx.StaticText(self.panelSkills, label='Number', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headScore = wx.StaticText(self.panelSkills, label='Score', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headDifficulty = wx.StaticText(self.panelSkills, label='Difficulty', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headAdj = wx.StaticText(self.panelSkills, label='adj', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headChance = wx.StaticText(self.panelSkills, label='Chance', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headRoll = wx.StaticText(self.panelSkills, label='Roll', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        headSkillResult = wx.StaticText(self.panelSkills, label='Result', style=wx.ALIGN_CENTER | wx.ST_NO_AUTORESIZE)
        self.buttonRollSkills = wx.Button(self.panelSkills, label='Roll All')
        self.buttonAddSkillsSet = wx.Button(self.panelSkills, label='Add Set')
        self.buttonDelSkillsSet = wx.Button(self.panelSkills, label='Del Set')
        self.sizerSkills.Add(headNumSkills)
        self.sizerSkills.Add(headScore)
        self.sizerSkills.Add(headDifficulty)
        self.sizerSkills.Add(headAdj)
        self.sizerSkills.Add(headChance)
        self.sizerSkills.Add(headRoll)
        self.sizerSkills.Add(headSkillResult)
        self.sizerSkills.Add(self.buttonRollSkills)
        self.sizerSkills.Add(self.buttonAddSkillsSet)
        self.sizerSkills.Add(self.buttonDelSkillsSet)
        self.panelSkills.SetSizer(self.sizerSkills)
        self.panelSkills.SetAutoLayout(True)
        self.AddSkillsSet(False)
        #
        self.SetSizer(self.sizerMain)
        self.SetAutoLayout(True)
        self.CreateStatusBar() # A Statusbar in the bottom of the window
        # Setting up the menu.
        filemenu= wx.Menu()
        filemenu.Append(ID_ABOUT, "&About"," Information about this program")
        filemenu.AppendSeparator()
        filemenu.Append(ID_EXIT,"E&xit"," Terminate the program")
        # Creating the menubar.
        menuBar = wx.MenuBar()
        menuBar.Append(filemenu,"&File") # Adding the "filemenu" to the MenuBar
        self.SetMenuBar(menuBar)  # Adding the MenuBar to the Frame content.
        wx.EVT_MENU(self, ID_ABOUT, self.OnAbout) # attach the menu-event ID_ABOUT to the
                                                  # method self.OnAbout
        wx.EVT_MENU(self, ID_EXIT, self.OnExit)   # attach the menu-event ID_EXIT to the
                                                  # method self.OnExit
        # Events
        self.buttonRoll.Bind(wx.EVT_LEFT_UP, self.RollAll)
        self.buttonAddSet.Bind(wx.EVT_LEFT_UP, self.AddDiceSet)
        self.buttonDelSet.Bind(wx.EVT_LEFT_UP, self.DelDiceSet)
        self.buttonRollSkills.Bind(wx.EVT_LEFT_UP, self.RollAllSkills)
        self.buttonAddSkillsSet.Bind(wx.EVT_LEFT_UP, self.AddSkillsSet)
        self.buttonDelSkillsSet.Bind(wx.EVT_LEFT_UP, self.DelSkillsSet)
        #
        self.Show(True)
    def RollAll(self, event):
        for diceSet in self.DiceSets:
            diceSet.UpdateSet(event)
    def AddDiceSet(self, event):
        diceSet = DiceSet(self.panelDice, 3 * len(self.DiceSets) + 3, 1)
        self.DiceSets.append(diceSet)
        self.Layout()
    def DelDiceSet(self, event):
        if (len(self.DiceSets) > 1):
          diceSet = self.DiceSets.pop()
          diceSet.Destroy()
          self.Layout()
    def RollAllSkills(self, event):
        for skillSet in self.SkillSets:
            skillSet.UpdateSet(event)
    def AddSkillsSet(self, event):
        skillSet = SkillSet(self.panelSkills, 7 * len(self.SkillSets) + 7, 1)
        self.SkillSets.append(skillSet)
        self.Layout()
    def DelSkillsSet(self, event):
        if (len(self.SkillSets) > 1):
          skillSet = self.SkillSets.pop()
          skillSet.Destroy()
          self.Layout()
    def OnAbout(self,e):
        d= wx.MessageDialog( self, " A dice rolling utility for Rune Master \n"
                            " by Tim Doty","About Dice Roller", wx.OK)
                            # Create a message dialog box
        d.ShowModal() # Shows it
        d.Destroy() # finally destroy it when finished.
    def OnExit(self,e):
        self.Close(True)  # Close the frame.

app = wx.App()
frame = MainWindow(None, wx.ID_ANY, "Die Roller")
app.MainLoop()

